# generated by datamodel-codegen:
#   filename:  solution-analyzer.schema.json
#   timestamp: 2024-06-26T17:45:07+00:00

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from allotropy.allotrope.models.shared.definitions.custom import (
    TQuantityValueCell,
    TQuantityValueDegreeCelsius,
    TQuantityValueGramPerLiter,
    TQuantityValueMicrometer,
    TQuantityValueMilliAbsorbanceUnit,
    TQuantityValueMilliliterPerLiter,
    TQuantityValueMillimeterOfMercury,
    TQuantityValueMillimolePerLiter,
    TQuantityValueMillionCellsPerMilliliter,
    TQuantityValueMilliOsmolesPerKilogram,
    TQuantityValueNanometer,
    TQuantityValuePercent,
    TQuantityValuePH,
    TQuantityValuePicogramPerMilliliter,
    TQuantityValueUnitless,
)
from allotropy.allotrope.models.shared.definitions.definitions import (
    TClass,
    TDatacube,
    TDateTimeStampValue,
    TStatisticDatumRole,
    TStringValue,
    TUnit,
)


@dataclass(kw_only=True)
class DiagnosticTraceDocumentItem:
    description: Any


@dataclass(kw_only=True)
class DiagnosticTraceAggregateDocument:
    diagnostic_trace_document: list[DiagnosticTraceDocumentItem] | None = None


@dataclass(kw_only=True)
class AdmCoreREC202403ManifestSchema:
    vocabulary: list[str]
    json_schemas: list[str]
    field_id: str | None = None
    field_type: str | None = None
    shapes: list[str] | None = None


@dataclass(kw_only=True)
class OrderedItem:
    field_index: int | None = None


@dataclass(kw_only=True)
class DataSourceDocumentItem:
    data_source_identifier: TStringValue
    data_source_feature: TStringValue
    field_index: int | None = None


@dataclass(kw_only=True)
class DataSourceAggregateDocument:
    data_source_document: list[DataSourceDocumentItem]


@dataclass(kw_only=True)
class ErrorDocumentItem:
    error: TStringValue
    error_feature: TStringValue | None = None


@dataclass(kw_only=True)
class ErrorAggregateDocument:
    error_document: list[ErrorDocumentItem] | None = None


@dataclass(kw_only=True)
class ProcessedDataDocumentItem32(OrderedItem):
    data_processing_document: dict[str, Any] | None = None
    data_source_aggregate_document: DataSourceAggregateDocument | None = None
    processed_data_identifier: TStringValue | None = None


@dataclass(kw_only=True)
class ProcessedDataAggregateDocument:
    processed_data_document: list[ProcessedDataDocumentItem32]


@dataclass(kw_only=True)
class StatisticsDocumentItem:
    statistical_feature: TClass


@dataclass(kw_only=True)
class StatisticsAggregateDocument:
    statistics_document: list[StatisticsDocumentItem] | None = None


@dataclass(kw_only=True)
class TIntegerValue1:
    field_type: TClass
    value: int


TIntegerValue = int | TIntegerValue1


@dataclass(kw_only=True)
class TQuantityValueModel:
    value: float
    unit: TUnit
    has_statistic_datum_role: TStatisticDatumRole | None = None
    field_type: TClass | None = None


@dataclass(kw_only=True)
class DeviceDocumentItem:
    device_type: TStringValue
    device_identifier: TStringValue | None = None
    model_number: TStringValue | None = None
    product_manufacturer: TStringValue | None = None
    brand_name: TStringValue | None = None
    equipment_serial_number: TStringValue | None = None
    firmware_version: TStringValue | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class DeviceSystemDocument:
    asset_management_identifier: TStringValue | None = None
    description: Any | None = None
    brand_name: TStringValue | None = None
    product_manufacturer: TStringValue | None = None
    device_identifier: TStringValue | None = None
    model_number: TStringValue | None = None
    equipment_serial_number: TStringValue | None = None
    firmware_version: TStringValue | None = None
    device_document: list[DeviceDocumentItem] | None = None


@dataclass(kw_only=True)
class DataSystemDocument:
    data_system_instance_identifier: TStringValue | None = None
    file_name: TStringValue | None = None
    UNC_path: TStringValue | None = None
    software_name: TStringValue | None = None
    software_version: TStringValue | None = None
    ASM_converter_name: TStringValue | None = None
    ASM_converter_version: TStringValue | None = None


@dataclass(kw_only=True)
class DeviceControlDocumentItem:
    device_type: TStringValue
    device_identifier: TStringValue | None = None
    detection_type: TStringValue | None = None
    product_manufacturer: TStringValue | None = None
    brand_name: TStringValue | None = None
    equipment_serial_number: TStringValue | None = None
    model_number: TStringValue | None = None
    firmware_version: TStringValue | None = None
    field_index: int | None = None
    detector_wavelength_setting: TQuantityValueNanometer | None = None
    detector_bandwidth_setting: TQuantityValueNanometer | None = None
    electronic_absorbance_wavelength_setting: TQuantityValueNanometer | None = None
    electronic_absorbance_bandwidth_setting: TQuantityValueNanometer | None = None
    electronic_absorbance_reference_bandwidth_setting: TQuantityValueNanometer | None = (
        None
    )
    electronic_absorbance_reference_wavelength_setting: TQuantityValueNanometer | None = (
        None
    )
    excitation_wavelength_setting: TQuantityValueNanometer | None = None


@dataclass(kw_only=True)
class DeviceControlAggregateDocument:
    device_control_document: list[DeviceControlDocumentItem]


@dataclass(kw_only=True)
class SampleDocument:
    sample_identifier: TStringValue
    description: Any | None = None
    batch_identifier: TStringValue | None = None
    sample_role_type: TClass | None = None
    written_name: TStringValue | None = None


@dataclass(kw_only=True)
class DataProcessingDocument:
    cell_type_processing_method: TStringValue | None = None
    cell_density_dilution_factor: TQuantityValueUnitless | None = None
    minimum_cell_diameter_setting: TQuantityValueMicrometer | None = None
    maximum_cell_diameter_setting: TQuantityValueMicrometer | None = None


@dataclass(kw_only=True)
class AnalyteDocument:
    analyte_name: TStringValue
    volume_concentration: TQuantityValueMilliliterPerLiter
    molar_concentration: TQuantityValueMillimolePerLiter
    mass_concentration: TQuantityValueGramPerLiter


@dataclass(kw_only=True)
class AnalyteDocument1:
    analyte_name: TStringValue
    molar_concentration: TQuantityValueMillimolePerLiter
    mass_concentration: TQuantityValueGramPerLiter
    volume_concentration: TQuantityValueMilliliterPerLiter | None = None


@dataclass(kw_only=True)
class AnalyteAggregateDocument:
    analyte_document: list[
        AnalyteDocument
        | AnalyteDocument1
        | AnalyteDocument10
        | AnalyteDocument100
        | AnalyteDocument102
        | AnalyteDocument103
        | AnalyteDocument104
    ]


@dataclass(kw_only=True)
class AnalyteDocument10:
    analyte_name: TStringValue
    mass_concentration: TQuantityValueGramPerLiter
    volume_concentration: TQuantityValueMilliliterPerLiter | None = None
    molar_concentration: TQuantityValueMillimolePerLiter | None = None


@dataclass(kw_only=True)
class AnalyteDocument100:
    analyte_name: TStringValue
    volume_concentration: TQuantityValueMilliliterPerLiter
    mass_concentration: TQuantityValueGramPerLiter
    molar_concentration: TQuantityValueMillimolePerLiter | None = None


@dataclass(kw_only=True)
class AnalyteDocument102:
    analyte_name: TStringValue
    volume_concentration: TQuantityValueMilliliterPerLiter
    molar_concentration: TQuantityValueMillimolePerLiter
    mass_concentration: TQuantityValueGramPerLiter | None = None


@dataclass(kw_only=True)
class AnalyteDocument103:
    analyte_name: TStringValue
    molar_concentration: TQuantityValueMillimolePerLiter
    volume_concentration: TQuantityValueMilliliterPerLiter | None = None
    mass_concentration: TQuantityValueGramPerLiter | None = None


@dataclass(kw_only=True)
class AnalyteDocument104:
    analyte_name: TStringValue
    volume_concentration: TQuantityValueMilliliterPerLiter
    molar_concentration: TQuantityValueMillimolePerLiter | None = None
    mass_concentration: TQuantityValueGramPerLiter | None = None


@dataclass(kw_only=True)
class CalculatedDataDocumentItem:
    calculated_data_name: TStringValue
    calculated_result: TQuantityValueModel
    data_source_aggregate_document: DataSourceAggregateDocument | None = None
    calculated_data_identifier: TStringValue | None = None
    calculation_description: TStringValue | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class CalculatedDataAggregateDocument:
    calculated_data_document: list[CalculatedDataDocumentItem]


@dataclass(kw_only=True)
class ImageDocumentItem:
    index: TIntegerValue | None = None
    experimental_data_identifier: TStringValue | None = None


@dataclass(kw_only=True)
class ImageAggregateDocument:
    image_document: list[ImageDocumentItem] | None = None


@dataclass(kw_only=True)
class MeasurementDocument16:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument17:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument18:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument19:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument20:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument21:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument22:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument23:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument24:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument25:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument26:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument27:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument28:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None


@dataclass(kw_only=True)
class MeasurementDocument29:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument30:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument31:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument48:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument49:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument50:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument51:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument52:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument53:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument54:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument55:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument56:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument57:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument58:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument59:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument60:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None


@dataclass(kw_only=True)
class MeasurementDocument61:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    absorbance: TQuantityValueMilliAbsorbanceUnit
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument62:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    detection_type: TStringValue | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class ProcessedDataDocumentItem:
    viability__cell_counter_: TQuantityValuePercent
    viable_cell_density__cell_counter_: TQuantityValueMillionCellsPerMilliliter
    data_processing_document: DataProcessingDocument | None = None
    data_source_aggregate_document: DataSourceAggregateDocument | None = None
    processed_data_identifier: TStringValue | None = None
    field_index: int | None = None
    total_cell_density__cell_counter_: TQuantityValueMillionCellsPerMilliliter | None = (
        None
    )
    dead_cell_density__cell_counter_: TQuantityValueMillionCellsPerMilliliter | None = (
        None
    )
    average_total_cell_diameter: TQuantityValueMicrometer | None = None
    average_live_cell_diameter__cell_counter_: TQuantityValueMicrometer | None = None
    average_dead_cell_diameter__cell_counter_: TQuantityValueMicrometer | None = None
    total_cell_diameter_distribution: TDatacube | None = None
    total_cell_count: TQuantityValueCell | None = None
    viable_cell_count: TQuantityValueCell | None = None
    dead_cell_count: TQuantityValueCell | None = None
    average_total_cell_circularity: TQuantityValueUnitless | None = None
    average_viable_cell_circularity: TQuantityValueUnitless | None = None


@dataclass(kw_only=True)
class ProcessedDataAggregateDocument1:
    processed_data_document: list[ProcessedDataDocumentItem]


@dataclass(kw_only=True)
class MeasurementDocument:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument1:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument2:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument3:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument4:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument5:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument6:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument7:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument8:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument9:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument10:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument11:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    pH: TQuantityValuePH
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument12:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None


@dataclass(kw_only=True)
class MeasurementDocument13:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument14:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument15:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    analyte_aggregate_document: AnalyteAggregateDocument
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument32:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument33:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument34:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument35:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    pH: TQuantityValuePH
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument36:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument37:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument38:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None


@dataclass(kw_only=True)
class MeasurementDocument39:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    pO2: TQuantityValueMillimeterOfMercury
    pCO2: TQuantityValueMillimeterOfMercury
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    carbon_dioxide_saturation: TQuantityValuePercent | None = None
    oxygen_saturation: TQuantityValuePercent | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument40:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument41:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument42:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    temperature: TQuantityValueDegreeCelsius | None = None


@dataclass(kw_only=True)
class MeasurementDocument43:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    pH: TQuantityValuePH
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    temperature: TQuantityValueDegreeCelsius | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument44:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    absorbance: TQuantityValueMilliAbsorbanceUnit
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None


@dataclass(kw_only=True)
class MeasurementDocument45:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    absorbance: TQuantityValueMilliAbsorbanceUnit
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    mass_concentration: TQuantityValuePicogramPerMilliliter | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument46:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    osmolality: TQuantityValueMilliOsmolesPerKilogram
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementDocument47:
    measurement_time: TDateTimeStampValue
    measurement_identifier: TStringValue
    device_control_aggregate_document: DeviceControlAggregateDocument
    sample_document: SampleDocument
    processed_data_aggregate_document: ProcessedDataAggregateDocument1
    detection_type: TStringValue | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    field_index: int | None = None


@dataclass(kw_only=True)
class MeasurementAggregateDocument:
    measurement_document: list[
        MeasurementDocument
        | MeasurementDocument1
        | MeasurementDocument10
        | MeasurementDocument11
        | MeasurementDocument12
        | MeasurementDocument13
        | MeasurementDocument14
        | MeasurementDocument15
        | MeasurementDocument16
        | MeasurementDocument17
        | MeasurementDocument18
        | MeasurementDocument19
        | MeasurementDocument2
        | MeasurementDocument20
        | MeasurementDocument21
        | MeasurementDocument22
        | MeasurementDocument23
        | MeasurementDocument24
        | MeasurementDocument25
        | MeasurementDocument26
        | MeasurementDocument27
        | MeasurementDocument28
        | MeasurementDocument29
        | MeasurementDocument3
        | MeasurementDocument30
        | MeasurementDocument31
        | MeasurementDocument32
        | MeasurementDocument33
        | MeasurementDocument34
        | MeasurementDocument35
        | MeasurementDocument36
        | MeasurementDocument37
        | MeasurementDocument38
        | MeasurementDocument39
        | MeasurementDocument4
        | MeasurementDocument40
        | MeasurementDocument41
        | MeasurementDocument42
        | MeasurementDocument43
        | MeasurementDocument44
        | MeasurementDocument45
        | MeasurementDocument46
        | MeasurementDocument47
        | MeasurementDocument48
        | MeasurementDocument49
        | MeasurementDocument5
        | MeasurementDocument50
        | MeasurementDocument51
        | MeasurementDocument52
        | MeasurementDocument53
        | MeasurementDocument54
        | MeasurementDocument55
        | MeasurementDocument56
        | MeasurementDocument57
        | MeasurementDocument58
        | MeasurementDocument59
        | MeasurementDocument6
        | MeasurementDocument60
        | MeasurementDocument61
        | MeasurementDocument62
        | MeasurementDocument7
        | MeasurementDocument8
        | MeasurementDocument9
    ]
    diagnostic_trace_aggregate_document: DiagnosticTraceAggregateDocument | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    image_aggregate_document: ImageAggregateDocument | None = None
    error_aggregate_document: ErrorAggregateDocument | None = None
    data_processing_time: TDateTimeStampValue | None = None


@dataclass(kw_only=True)
class SolutionAnalyzerDocumentItem:
    measurement_aggregate_document: MeasurementAggregateDocument
    analyst: TStringValue | None = None
    submitter: TStringValue | None = None


@dataclass(kw_only=True)
class SolutionAnalyzerAggregateDocument:
    solution_analyzer_document: list[SolutionAnalyzerDocumentItem]
    device_system_document: DeviceSystemDocument | None = None
    processed_data_aggregate_document: ProcessedDataAggregateDocument | None = None
    calculated_data_aggregate_document: CalculatedDataAggregateDocument | None = None
    statistics_aggregate_document: StatisticsAggregateDocument | None = None
    data_system_document: DataSystemDocument | None = None


@dataclass(kw_only=True)
class Model:
    field_asm_manifest: AdmCoreREC202403ManifestSchema | str
    solution_analyzer_aggregate_document: SolutionAnalyzerAggregateDocument | None = (
        None
    )
